import { NextFunction, Request, Response } from "express";
import prisma from "../../database/dbclient";
import { z } from "zod";

export const getAllRtwOfCompany = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  const { company_id } = req.params;
  const { page = 1, limit = 12 } = req.query;

  const pageNum = parseInt(page as string);
  const limitNum = parseInt(limit as string);

  const skip = (pageNum - 1) * limitNum;

  try {
    const rtw = await prisma.rtw_db.findMany({
      where: { company_id: parseInt(company_id) },
      include: {
        employee: {
          include: {
            user: true,
          },
        },
      },
      take: limitNum,
      skip: skip,
    });

    const total_rows = await prisma.rtw_db.count({
      where: { company_id: Number.parseInt(company_id) },
    });
    const total_page = Math.ceil(total_rows / limitNum);

    // Parse each JSON string back to an object
    const parsedDataList = rtw.map((data) => ({
      ...data,
      body: data.body ? JSON.parse(data.body) : null,
    }));

    res.status(200).json({
      total_page,
      data_count: rtw.length,
      next_page: pageNum < total_page ? pageNum + 1 : undefined,
      previous_page: pageNum > 1 ? pageNum - 1 : undefined,
      data: parsedDataList,
    });
  } catch (error) {
    next(error);
  }
};

// Insert a new message
export const insertRtw = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    // Validate request body with Zod
    const jsonData = req.body;

    const { employee_id, company_id, create_by } = req.body;
    // Insert data into the database
    const newMessage = await prisma.rtw_db.create({
      data: {
        employee_id: parseInt(employee_id),
        body: JSON.stringify(jsonData),
        company_id: parseInt(company_id),
        created_by: parseInt(create_by),
      },
    });

    res
      .status(201)
      .json({ message: "Message created successfully", data: newMessage });
  } catch (error) {
    next(error);
  }
};

export const updateRTW = async (
  req: Request,
  res: Response,
  next: NextFunction
) => {
  try {
    const id = Number.parseInt(req.params.id);
    // Validate request body with Zod
    const jsonData = req.body;

    const { employee_id, company_id, create_by } = req.body;
    // Insert data into the database
    const newMessage = await prisma.rtw_db.update({
      where: {
        id,
      },
      data: {
        body: JSON.stringify(jsonData),
      },
    });

    res
      .status(201)
      .json({ message: "Message created successfully", data: newMessage });
  } catch (error) {
    console.error("PATCH > Update RTW > Failed to update RTW >", error);
    next(error);
  }
};
